@@ -24,6 +24,9 @@
 
 #include <gtk/gtk.h>
 
-void gnome_theme_installer_run (GtkWidget *parent, gchar *filename);
+void gnome_theme_install_from_uri (gchar * filename, GtkWindow *parent);
+
+void gnome_theme_installer_run (GtkWindow *parent, gchar *filename);
+void gnome_theme_installer_run_cb (GtkWidget *button, GtkWindow *parent_window);
 
 #endif /* GNOME_THEME_INSTALLER_H */
