@@ -78,7 +78,7 @@ dialog_button_clicked_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *c
 {
 	if (response_id == GTK_RESPONSE_HELP)
 		capplet_help (GTK_WINDOW (dialog),
-			"wgoscustdesk.xml",
+			"user-guide.xml",
 			"goscustuserinter-2");
 	else
 		gtk_main_quit ();
@@ -89,7 +89,7 @@ create_dialog (void)
 {
   GladeXML *dialog;
 
-  dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-ui-properties.glade", "gnome_ui_properties_dialog", NULL);
+  dialog = glade_xml_new (GNOMECC_GLADE_DIR "/gnome-ui-properties.glade", "gnome_ui_properties_dialog", NULL);
   return dialog;
 }
 
@@ -98,28 +98,28 @@ show_handlebar (GladeXML *dialog, gboolean show)
 {
   GtkWidget *handlebox;
   GtkWidget *toolbar;
-  GtkWidget *frame;
+  GtkWidget *align;
 
   handlebox = WID ("toolbar_handlebox");
   toolbar = WID ("toolbar_toolbar");
-  frame = WID ("toolbar_frame");
+  align = WID ("toolbar_align");
 
   g_object_ref (handlebox);
   g_object_ref (toolbar);
 
-  if (GTK_BIN (frame)->child)
-    gtk_container_remove (GTK_CONTAINER (frame), GTK_BIN (frame)->child);
+  if (GTK_BIN (align)->child)
+    gtk_container_remove (GTK_CONTAINER (align), GTK_BIN (align)->child);
   if (GTK_BIN (handlebox)->child)
     gtk_container_remove (GTK_CONTAINER (handlebox), GTK_BIN (handlebox)->child);
   if (show)
     {
-      gtk_container_add (GTK_CONTAINER (frame), handlebox);
+      gtk_container_add (GTK_CONTAINER (align), handlebox);
       gtk_container_add (GTK_CONTAINER (handlebox), toolbar);
       g_object_unref (handlebox);
     }
   else
     {
-      gtk_container_add (GTK_CONTAINER (frame), toolbar);
+      gtk_container_add (GTK_CONTAINER (align), toolbar);
     }
   g_object_unref (toolbar);
 
@@ -223,6 +223,7 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
   GtkWidget *widget;
   GObject *peditor;
   char *toolbar_style;
+  GConfClient *client = gconf_client_get_default ();
 
   peditor = gconf_peditor_new_boolean
     (changeset, "/desktop/gnome/interface/toolbar_detachable", WID ("detachable_toolbars_toggle"), NULL);
@@ -230,6 +231,9 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
     "value_changed",
     G_CALLBACK (toolbar_detachable_cb), dialog);
 
+  peditor = gconf_peditor_new_boolean
+    (changeset, "/desktop/gnome/interface/can_change_accels", WID ("menu_accel_toggle"), NULL);
+
   peditor = gconf_peditor_new_boolean
     (changeset, "/desktop/gnome/interface/menus_have_icons", WID ("menu_icons_toggle"), NULL);
   g_signal_connect (peditor,
@@ -237,7 +241,7 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
     G_CALLBACK (menus_have_icons_cb), dialog);
   
   set_have_icons (dialog, 
-		  gconf_client_get_bool (gconf_client_get_default (),
+		  gconf_client_get_bool (client,
 					 "/desktop/gnome/interface/menus_have_icons",
 					 NULL));
 
@@ -261,26 +265,28 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	G_CALLBACK (dialog_button_clicked_cb), changeset);
 
   show_handlebar (dialog, 
-		  gconf_client_get_bool (gconf_client_get_default (),
+		  gconf_client_get_bool (client,
 					 "/desktop/gnome/interface/toolbar_detachable",
 					 NULL));
 
-  toolbar_style = gconf_client_get_string (gconf_client_get_default (),
+  toolbar_style = gconf_client_get_string (client,
 					   "/desktop/gnome/interface/toolbar_style", 
 					   NULL);
-  
+  g_object_unref (client);
+
   set_toolbar_style (dialog, toolbar_style);
 
   g_free (toolbar_style);
 
   gtk_window_set_resizable (GTK_WINDOW (widget), FALSE);
-  capplet_set_icon (widget, "behavior-capplet.png");
+  capplet_set_icon (widget, "gnome-settings-ui-behavior");
   gtk_widget_show_all (widget);
 }
 
 int
 main (int argc, char **argv)
 {
+  GnomeProgram   *program;
   GConfClient    *client;
   GConfChangeSet *changeset = NULL;
   GladeXML       *dialog;
@@ -289,17 +295,19 @@ main (int argc, char **argv)
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  gnome_program_init ("gnome-ui-properties", VERSION,
-		      LIBGNOMEUI_MODULE, argc, argv,
-		      GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
-		      NULL);
+  program = gnome_program_init ("gnome-ui-properties", VERSION,
+				LIBGNOMEUI_MODULE, argc, argv,
+				GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
+				NULL);
 
   client = gconf_client_get_default ();
   gconf_client_add_dir (client, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-
+  g_object_unref (client);
   dialog = create_dialog ();
   setup_dialog (dialog, changeset);
   gtk_main ();
 
+  g_object_unref (dialog);
+  g_object_unref (program);
   return 0;
 }
