@@ -116,9 +116,9 @@ acme_volume_gstreamer_set_mute (AcmeVolume *vol, gboolean val)
 			GstMixerTrack *track = GST_MIXER_TRACK (t->data);
 			gint *volumes, n;
 			gdouble scale = (track->max_volume - track->min_volume) / 100.0;
-			gint vol = (gint) self->_priv->volume * scale + track->min_volume;
+			gint vol = (gint) (self->_priv->volume * scale + track->min_volume + 0.5);
 
-			volumes = g_new0 (gint, track->num_channels);
+			volumes = g_new (gint, track->num_channels);
 			for (n = 0; n < track->num_channels; n++)
 				volumes[n] = vol;
 			gst_mixer_set_volume (self->_priv->mixer, track, volumes);
@@ -133,7 +133,7 @@ static void
 update_state (AcmeVolumeGStreamer * self)
 {
 	gint *volumes, n;
-	gdouble vol;
+	gdouble vol = 0;
 	GstMixerTrack *track = GST_MIXER_TRACK(self->_priv->mixer_tracks->data);
 
 	/* update mixer by getting volume */
@@ -179,7 +179,7 @@ acme_volume_gstreamer_get_volume (AcmeVolume *vol)
 
 	acme_volume_gstreamer_close (self);
 
-	return (gint) self->_priv->volume;
+	return (gint) (self->_priv->volume + 0.5);
 }
 
 static void
@@ -198,9 +198,9 @@ acme_volume_gstreamer_set_volume (AcmeVolume *vol, int val)
 		GstMixerTrack *track = GST_MIXER_TRACK (t->data);
 		gint *volumes, n;
 		gdouble scale = (track->max_volume - track->min_volume) / 100.0;
-		gint vol = (gint) val * scale + track->min_volume;
+		gint vol = (gint) (val * scale + track->min_volume + 0.5);
 
-		volumes = g_new0 (gint, track->num_channels);
+		volumes = g_new (gint, track->num_channels);
 		for (n = 0; n < track->num_channels; n++)
 			volumes[n] = vol;
 		gst_mixer_set_volume (self->_priv->mixer, track, volumes);
@@ -350,7 +350,7 @@ acme_volume_gstreamer_open (AcmeVolumeGStreamer *vol)
 				GstMixerTrack *track = GST_MIXER_TRACK (m->data);
 
 				if (GST_MIXER_TRACK_HAS_FLAG (track, GST_MIXER_TRACK_MASTER)) {
-					self->_priv->mixer_tracks = g_list_append (self->_priv->mixer_tracks, track);
+					self->_priv->mixer_tracks = g_list_append (self->_priv->mixer_tracks, g_object_ref (track));
 					break;
 				}
 			}
