@@ -1,5 +1,7 @@
-/* 
- * Copyright (C) 2001,2002,2003 Bastien Nocera <hadess@hadess.net>
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ * Copyright (C) 2001-2003 Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) 2006      William Jon McCann <mccann@jhu.edu>
  *
  * gnome-settings-multimedia-keys.c
  *
@@ -30,17 +32,17 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
-#include <glade/glade.h>
 #include <gconf/gconf-client.h>
 
 #include "eggaccelerators.h"
 
 #include "actions/acme.h"
 #include "actions/acme-volume.h"
+#include "gsd-media-keys-window.h"
+#include "gnome-settings-dbus.h"
 
-#define DIALOG_TIMEOUT 1000     /* dialog timeout in ms */
 #define VOLUME_STEP 6           /* percents for one volume button press */
-                                                                                
+
 /* we exclude shift, GDK_CONTROL_MASK and GDK_MOD1_MASK since we know what
    these modifiers mean
    these are the mods whose combinations are bound by the keygrabbing code */
@@ -50,22 +52,19 @@
  * for these set */
 #define USED_MODS (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK)
 
-typedef struct {
+typedef struct Acme Acme;
+
+struct Acme {
 	AcmeVolume *volobj;
-	GladeXML *xml;
 	GtkWidget *dialog;
 	GConfClient *conf_client;
-	guint dialog_timeout;
 
 	/* Multihead stuff */
 	GdkScreen *current_screen;
 	GSList *screens;
-} Acme;
 
-enum {
-	ICON_MUTED,
-	ICON_LOUD,
-	ICON_EJECT,
+	/* GnomeSettingsServer */
+	GObject *server;
 };
 
 static void
@@ -93,17 +92,17 @@ execute (char *cmd, gboolean sync)
 	gboolean retval;
 	gchar **argv;
 	gint argc;
-	
+
 	retval = FALSE;
 
 	if (g_shell_parse_argv (cmd, &argc, &argv, NULL)) {
 		if (sync != FALSE) {
-			retval = g_spawn_sync (g_get_home_dir (), 
+			retval = g_spawn_sync (g_get_home_dir (),
 			                       argv, NULL, G_SPAWN_SEARCH_PATH,
 			                       NULL, NULL, NULL, NULL, NULL, NULL);
 		}
 		else {
-			retval = g_spawn_async (g_get_home_dir (), 
+			retval = g_spawn_async (g_get_home_dir (),
 			                        argv, NULL, G_SPAWN_SEARCH_PATH,
 			                        NULL, NULL, NULL, NULL);
 		}
@@ -137,40 +136,11 @@ do_sleep_action (char *cmd1, char *cmd2)
 	}
 }
 
-static char *images[] = {
-	PIXMAPSDIR "/gnome-speakernotes-muted.png",
-	PIXMAPSDIR "/gnome-speakernotes.png",
-	PIXMAPSDIR "/acme-eject.png",
-};
-
-static void
-acme_image_set (Acme *acme, int icon)
-{
-	GtkWidget *image;
-
-	image = glade_xml_get_widget (acme->xml, "image1");
-	g_return_if_fail (image != NULL);
-
-	if (icon > ICON_EJECT)
-		g_assert_not_reached ();
-
-	gtk_image_set_from_file (GTK_IMAGE(image), images[icon]);
-}
-
 static void
 dialog_init (Acme *acme)
 {
-	if (acme->xml == NULL) {
-		glade_gnome_init ();
-		acme->xml = glade_xml_new (DATADIR "/control-center-2.0/interfaces/acme.glade", NULL, NULL);
-
-		if (acme->xml == NULL) {
-			acme_error (_("Couldn't load the Glade file.\n"
-				      "Make sure that this daemon is properly installed."));
-			return;
-		}
-		acme->dialog = glade_xml_get_widget (acme->xml, "dialog");
-		acme_image_set (acme, ICON_LOUD);
+	if (acme->dialog == NULL) {
+		acme->dialog = gsd_media_keys_window_new ();
 	}
 }
 
@@ -227,35 +197,6 @@ grab_key (Acme *acme, Key *key, gboolean grab)
 	}
 }
 
-static void                                                                     
-unhookup_keysym (int keycode)
-{
-	char *command;
-
-	if (keycode <= 0)
-		return;
-
-	command = g_strdup_printf ("xmodmap -e \"keycode %d = \"", keycode);
-	g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
-	g_free (command);
-}                                                                               
-
-static gboolean
-hookup_keysym (int keycode, const char *keysym)
-{
-	char *command;
-
-	if (keycode <= 0)
-		return TRUE;
-
-	command = g_strdup_printf ("xmodmap -e \"keycode %d = %s\"",
-				   keycode, keysym);
-	g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
-	g_free (command);
-
-	return FALSE;
-}
-
 static gboolean
 is_valid_shortcut (const char *string)
 {
@@ -277,7 +218,7 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 	g_return_if_fail (entry->key != NULL);
 
 	/* Find the key that was modified */
-	for (i = 0; i < PLAY_KEY; i++)
+	for (i = 0; i < HANDLED_KEYS; i++)
 	{
 		if (strcmp (entry->key, keys[i].gconf_key) == 0)
 		{
@@ -321,61 +262,6 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 
 	if (found != FALSE)
 		return;
-
-	for (i = PLAY_KEY; i < HANDLED_KEYS; i++)
-	{
-		if (strcmp (entry->key, keys[i].gconf_key) == 0)
-		{
-			char *tmp;
-			Key *key;
-
-			if (keys[i].key != NULL)
-				unhookup_keysym (keys[i].key->keycode);
-
-			g_free (keys[i].key);
-			keys[i].key = NULL;
-
-			tmp = gconf_client_get_string (acme->conf_client,
-						       keys[i].gconf_key, NULL);
-
-			if (is_valid_shortcut (tmp) == FALSE)
-			{
-				g_free (tmp);
-				break;
-			}
-
-			key = g_new0 (Key, 1);
-			if (egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state) == FALSE
-			    || key->keycode == 0)
-			{
-				g_free (tmp);
-				g_free (key);
-				break;
-			}
-
-			switch (keys[i].key_type) {
-			case PLAY_KEY:
-				hookup_keysym (key->keycode, "XF86AudioPlay");
-				break;
-			case PAUSE_KEY:
-				hookup_keysym (key->keycode, "XF86AudioPause");
-				break; 
-			case STOP_KEY:
-				hookup_keysym (key->keycode, "XF86AudioStop");
-				break;
-			case PREVIOUS_KEY:
-				hookup_keysym (key->keycode, "XF86AudioPrev");
-				break;
-			case NEXT_KEY:                                                
-				hookup_keysym (key->keycode, "XF86AudioNext");
-				break;
-			}
-
-			keys[i].key = key;
-
-			g_free (tmp);
-		}
-	}
 }
 
 static void
@@ -409,7 +295,7 @@ init_kbd (Acme *acme)
 {
 	int i;
 
-	for (i = 0; i < PLAY_KEY; i++)
+	for (i = 0; i < HANDLED_KEYS; i++)
 	{
 		char *tmp;
 		Key *key;
@@ -448,60 +334,6 @@ init_kbd (Acme *acme)
 
 		grab_key (acme, key, TRUE);
 	}
-
-	for (i = PLAY_KEY; i < HANDLED_KEYS; i++)
-	{
-		char *tmp;
-		Key *key;
-
-		gconf_client_notify_add (acme->conf_client,
-					 keys[i].gconf_key, update_kbd_cb,
-					 acme, NULL, NULL);
-
-		tmp = gconf_client_get_string (acme->conf_client,
-					       keys[i].gconf_key, NULL);
-		if (!is_valid_shortcut (tmp)) {
-			g_free (tmp);
-			continue;                                           
-		}                                               
-
-		key = g_new0 (Key, 1);
-		if (egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state) == FALSE
-		    || key->keycode == 0)
-		{                                       
-			g_free (tmp);
-			g_free (key);
-			continue;
-		}                                               
-		g_free (tmp);
-
-		keys[i].key = key;
-		switch (keys[i].key_type) {
-		case PLAY_KEY:
-			hookup_keysym (keys[i].key->keycode, "XF86AudioPlay");
-			break;
-		case PAUSE_KEY:
-			hookup_keysym (keys[i].key->keycode, "XF86AudioPause");
-			break;
-		case STOP_KEY:
-			hookup_keysym (keys[i].key->keycode, "XF86AudioStop");
-			break;
-		case PREVIOUS_KEY:
-			hookup_keysym (keys[i].key->keycode, "XF86AudioPrev");
-			break;
-		case NEXT_KEY:
-			hookup_keysym (keys[i].key->keycode, "XF86AudioNext");
-			break;
-		}
-	}
-}
-
-static gboolean
-dialog_hide (Acme *acme)
-{
-	gtk_widget_hide (acme->dialog);
-	acme->dialog_timeout = 0;
-	return FALSE;
 }
 
 static void
@@ -535,7 +367,7 @@ dialog_show (Acme *acme)
 		monitor = gdk_screen_get_monitor_at_point (acme->current_screen,
 							   pointer_x, pointer_y);
 	}
-		
+
 	gdk_screen_get_monitor_geometry (acme->current_screen, monitor,
 					 &geometry);
 
@@ -550,9 +382,6 @@ dialog_show (Acme *acme)
 	gtk_widget_show (acme->dialog);
 
 	gdk_display_sync (gdk_screen_get_display (acme->current_screen));
-
-	acme->dialog_timeout = gtk_timeout_add (DIALOG_TIMEOUT,
-			(GtkFunction) dialog_hide, acme);
 }
 
 static void
@@ -655,40 +484,26 @@ do_exit_action (Acme *acme)
 static void
 do_eject_action (Acme *acme)
 {
-	GtkWidget *progress;
 	char *command;
 
-	if (acme->dialog_timeout != 0)
-	{
-		gtk_timeout_remove (acme->dialog_timeout);
-		acme->dialog_timeout = 0;
-	}
-
 	dialog_init (acme);
-	progress = glade_xml_get_widget (acme->xml, "progressbar");
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress),
-			(double) 0);
-	gtk_widget_set_sensitive (progress, FALSE);
-
-	acme_image_set (acme, ICON_EJECT);
+	gsd_media_keys_window_set_action (GSD_MEDIA_KEYS_WINDOW (acme->dialog),
+					  GSD_MEDIA_KEYS_WINDOW_ACTION_EJECT);
 	dialog_show (acme);
 
 	command = gconf_client_get_string (acme->conf_client,
-			GCONF_MISC_DIR "/eject_command", NULL);
+					   GCONF_MISC_DIR "/eject_command", NULL);
 	if ((command != NULL) && (strcmp (command, "") != 0))
-		execute (command, TRUE);
+		execute (command, FALSE);
 	else
-		execute ("eject", TRUE);
+		execute ("eject", FALSE);
 
 	g_free (command);
-
-	gtk_widget_set_sensitive (progress, TRUE);
 }
 
 static void
 do_sound_action (Acme *acme, int type)
 {
-	GtkWidget *progress;
 	gboolean muted;
 	int vol;
 	int vol_step;
@@ -697,23 +512,18 @@ do_sound_action (Acme *acme, int type)
 		return;
 
 	vol_step = gconf_client_get_int (acme->conf_client,
-			GCONF_MISC_DIR "/volume_step", NULL);
+					 GCONF_MISC_DIR "/volume_step", NULL);
 
 	if (vol_step == 0)
 		vol_step = VOLUME_STEP;
 
-	if (acme->dialog_timeout != 0)
-	{
-		gtk_timeout_remove (acme->dialog_timeout);
-		acme->dialog_timeout = 0;
-	}
-
+	/* FIXME: this is racy */
 	vol = acme_volume_get_volume (acme->volobj);
 	muted = acme_volume_get_mute (acme->volobj);
 
 	switch (type) {
 	case MUTE_KEY:
-		acme_volume_mute_toggle(acme->volobj);
+		acme_volume_mute_toggle (acme->volobj);
 		break;
 	case VOLUME_DOWN_KEY:
 		if (muted)
@@ -733,20 +543,28 @@ do_sound_action (Acme *acme, int type)
 		break;
 	}
 
-	muted = acme_volume_get_mute(acme->volobj);
-	dialog_init (acme);
-	acme_image_set (acme, muted ? ICON_MUTED : ICON_LOUD);
-
+	muted = acme_volume_get_mute (acme->volobj);
 	vol = acme_volume_get_volume (acme->volobj);
 
-	progress = glade_xml_get_widget (acme->xml, "progressbar");
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress),
-			(double) vol / 100);
-
+	/* FIXME: AcmeVolume should probably emit signals
+	   instead of doing it like this */
+	dialog_init (acme);
+	gsd_media_keys_window_set_volume_muted (GSD_MEDIA_KEYS_WINDOW (acme->dialog),
+						muted);
+	gsd_media_keys_window_set_volume_level (GSD_MEDIA_KEYS_WINDOW (acme->dialog),
+						vol);
+	gsd_media_keys_window_set_action (GSD_MEDIA_KEYS_WINDOW (acme->dialog),
+					  GSD_MEDIA_KEYS_WINDOW_ACTION_VOLUME);
 	dialog_show (acme);
 }
 
-static void
+static gboolean
+do_multimedia_player_action (Acme *acme, const gchar *key)
+{
+	return gnome_settings_server_media_player_key_pressed (acme->server, key);
+}
+
+static gboolean
 do_action (Acme *acme, int type)
 {
 	gchar *cmd;
@@ -789,9 +607,26 @@ do_action (Acme *acme, int type)
 	case WWW_KEY:
 		do_www_action (acme, NULL);
 		break;
+	case PLAY_KEY:
+		return do_multimedia_player_action (acme, "Play");
+		break;
+	case PAUSE_KEY:
+		return do_multimedia_player_action (acme, "Pause");
+		break;
+	case STOP_KEY:
+		return do_multimedia_player_action (acme, "Stop");
+		break;
+	case PREVIOUS_KEY:
+		return do_multimedia_player_action (acme, "Previous");
+		break;
+	case NEXT_KEY:
+		return do_multimedia_player_action (acme, "Next");
+		break;
 	default:
 		g_assert_not_reached ();
 	}
+
+	return FALSE;
 }
 
 static GdkScreen *
@@ -825,10 +660,15 @@ acme_filter_events (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 	guint keycode, state;
 	int i;
 
+	/* verify we have a key event */
+	if (xev->xany.type != KeyPress &&
+			xev->xany.type != KeyRelease)
+		return GDK_FILTER_CONTINUE;
+
 	keycode = xev->xkey.keycode;
 	state = xev->xkey.state;
 
-	for (i = 0; i < PLAY_KEY; i++)
+	for (i = 0; i < HANDLED_KEYS; i++)
 	{
 		if (keys[i].key == NULL)
 			continue;
@@ -851,8 +691,10 @@ acme_filter_events (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 			acme->current_screen = acme_get_screen_from_event
 				(acme, xanyev);
 
-			do_action (acme, keys[i].key_type);
-			return GDK_FILTER_REMOVE;
+			if (do_action (acme, keys[i].key_type) == FALSE)
+				return GDK_FILTER_REMOVE;
+			else
+				return GDK_FILTER_CONTINUE;
 		}
 	}
 
@@ -863,25 +705,25 @@ void
 gnome_settings_multimedia_keys_init (GConfClient *client)
 {
 }
+
 void
 gnome_settings_multimedia_keys_load (GConfClient *client)
 {
 	GSList *l;
 	Acme   *acme;
-	GError *err = NULL;
 
 	acme = g_new0 (Acme, 1);
-	acme->xml = NULL;
 
 	acme->conf_client = client;
 	gconf_client_add_dir (acme->conf_client,
-		GCONF_BINDING_DIR,
-		GCONF_CLIENT_PRELOAD_ONELEVEL,
-		NULL);
+			      GCONF_BINDING_DIR,
+			      GCONF_CLIENT_PRELOAD_ONELEVEL,
+			      NULL);
 
 	init_screens (acme);
 	init_kbd (acme);
-	acme->dialog_timeout = 0;
+
+	acme->server = gnome_settings_server_get ();
 
 	/* initialise Volume handler */
 	acme->volobj = acme_volume_new();
