@@ -1,7 +1,7 @@
 /*
  * This file is part of libtile.
  *
- * Copyright (c) 2006 Novell, Inc.
+ * Copyright (c) 2006, 2007 Novell, Inc.
  *
  * Libtile is free software; you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -32,6 +32,7 @@
 
 #include "slab-gnome-util.h"
 #include "gnome-utils.h"
+#include "libslab-utils.h"
 #include "recent-files.h"
 
 #define GCONF_SEND_TO_CMD_KEY       "/desktop/gnome/applications/main-menu/file-area/file_send_to_cmd"
@@ -48,21 +49,26 @@ static void load_image (DocumentTile *);
 
 static GtkWidget *create_header (const gchar *);
 static GtkWidget *create_subheader (const gchar *);
+static void update_user_list_menu_item (DocumentTile *);
 
 static void header_size_allocate_cb (GtkWidget *, GtkAllocation *, gpointer);
 
-static void open_with_default_trigger (Tile *, TileEvent *, TileAction *);
+static void open_with_default_trigger    (Tile *, TileEvent *, TileAction *);
 static void open_in_file_manager_trigger (Tile *, TileEvent *, TileAction *);
-static void rename_trigger (Tile *, TileEvent *, TileAction *);
-static void move_to_trash_trigger (Tile *, TileEvent *, TileAction *);
-static void delete_trigger (Tile *, TileEvent *, TileAction *);
-static void send_to_trigger (Tile *, TileEvent *, TileAction *);
+static void rename_trigger               (Tile *, TileEvent *, TileAction *);
+static void move_to_trash_trigger        (Tile *, TileEvent *, TileAction *);
+static void delete_trigger               (Tile *, TileEvent *, TileAction *);
+static void user_docs_trigger            (Tile *, TileEvent *, TileAction *);
+static void send_to_trigger              (Tile *, TileEvent *, TileAction *);
 
 static void rename_entry_activate_cb (GtkEntry *, gpointer);
 static gboolean rename_entry_key_release_cb (GtkWidget *, GdkEventKey *, gpointer);
 
 static void gconf_enable_delete_cb (GConfClient *, guint, GConfEntry *, gpointer);
 
+static void docs_store_monitor_cb (GnomeVFSMonitorHandle *, const gchar *, const gchar *,
+                                   GnomeVFSMonitorEventType, gpointer);
+
 typedef struct
 {
 	gchar *basename;
@@ -79,6 +85,9 @@ typedef struct
 	gboolean delete_enabled;
 	guint gconf_conn_id;
 
+	gboolean is_in_user_list;
+
+	GnomeVFSMonitorHandle *user_monitor;
 	MainMenuRecentMonitor *recent_monitor;
 } DocumentTilePrivate;
 
@@ -174,8 +183,8 @@ document_tile_new (const gchar *in_uri, const gchar *mime_type, time_t modified)
 
 	document_tile_private_setup (this);
 
-	TILE (this)->actions = g_new0 (TileAction *, 6);
-	TILE (this)->n_actions = 6;
+	TILE (this)->actions = g_new0 (TileAction *, DOCUMENT_TILE_ACTION_NUM_OF_ACTIONS);
+	TILE (this)->n_actions = DOCUMENT_TILE_ACTION_NUM_OF_ACTIONS;
 
 	menu_ctnr = GTK_CONTAINER (TILE (this)->context_menu);
 
@@ -202,11 +211,6 @@ document_tile_new (const gchar *in_uri, const gchar *mime_type, time_t modified)
 
 	gtk_container_add (menu_ctnr, menu_item);
 
-	/* insert separator */
-
-	menu_item = gtk_separator_menu_item_new ();
-	gtk_container_add (menu_ctnr, menu_item);
-
 	/* make open in nautilus action */
 
 	action = tile_action_new (TILE (this), open_in_file_manager_trigger,
@@ -232,35 +236,6 @@ document_tile_new (const gchar *in_uri, const gchar *mime_type, time_t modified)
 	menu_item = GTK_WIDGET (tile_action_get_menu_item (action));
 	gtk_container_add (menu_ctnr, menu_item);
 
-	/* insert separator */
-
-	menu_item = gtk_separator_menu_item_new ();
-	gtk_container_add (menu_ctnr, menu_item);
-
-	/* make move to trash action */
-
-	action = tile_action_new (TILE (this), move_to_trash_trigger, _("Move to Trash"), 0);
-	TILE (this)->actions[DOCUMENT_TILE_ACTION_MOVE_TO_TRASH] = action;
-
-	menu_item = GTK_WIDGET (tile_action_get_menu_item (action));
-	gtk_container_add (menu_ctnr, menu_item);
-
-	/* make delete action */
-
-	if (priv->delete_enabled)
-	{
-		action = tile_action_new (TILE (this), delete_trigger, _("Delete"), 0);
-		TILE (this)->actions[DOCUMENT_TILE_ACTION_DELETE] = action;
-
-		menu_item = GTK_WIDGET (tile_action_get_menu_item (action));
-		gtk_container_add (menu_ctnr, menu_item);
-	}
-
-	/* insert separator */
-
-	menu_item = gtk_separator_menu_item_new ();
-	gtk_container_add (menu_ctnr, menu_item);
-
 	/* make send to action */
 
 	/* Only allow Send To for local files, ideally this would use something
@@ -286,6 +261,41 @@ document_tile_new (const gchar *in_uri, const gchar *mime_type, time_t modified)
 
 	gtk_container_add (menu_ctnr, menu_item);
 
+	/* make "add/remove to favorites" action */
+
+	action = tile_action_new (TILE (this), user_docs_trigger, NULL, 0);
+	TILE (this)->actions [DOCUMENT_TILE_ACTION_UPDATE_MAIN_MENU] = action;
+
+	update_user_list_menu_item (this);
+
+	menu_item = GTK_WIDGET (tile_action_get_menu_item (action));
+
+	gtk_container_add (menu_ctnr, menu_item);
+
+	/* insert separator */
+
+	menu_item = gtk_separator_menu_item_new ();
+	gtk_container_add (menu_ctnr, menu_item);
+
+	/* make move to trash action */
+
+	action = tile_action_new (TILE (this), move_to_trash_trigger, _("Move to Trash"), 0);
+	TILE (this)->actions[DOCUMENT_TILE_ACTION_MOVE_TO_TRASH] = action;
+
+	menu_item = GTK_WIDGET (tile_action_get_menu_item (action));
+	gtk_container_add (menu_ctnr, menu_item);
+
+	/* make delete action */
+
+	if (priv->delete_enabled)
+	{
+		action = tile_action_new (TILE (this), delete_trigger, _("Delete"), 0);
+		TILE (this)->actions[DOCUMENT_TILE_ACTION_DELETE] = action;
+
+		menu_item = GTK_WIDGET (tile_action_get_menu_item (action));
+		gtk_container_add (menu_ctnr, menu_item);
+	}
+
 	gtk_widget_show_all (GTK_WIDGET (TILE (this)->context_menu));
 
 	load_image (this);
@@ -337,6 +347,9 @@ document_tile_private_setup (DocumentTile *tile)
 
 	g_object_unref (client);
 
+	priv->is_in_user_list = libslab_user_docs_store_has_uri (TILE (tile)->uri);
+
+	priv->user_monitor   = libslab_add_docs_monitor (docs_store_monitor_cb, tile);
 	priv->recent_monitor = main_menu_recent_monitor_new ();
 }
 
@@ -353,6 +366,9 @@ document_tile_init (DocumentTile *tile)
 	priv->image_is_broken = TRUE;
 	priv->delete_enabled = FALSE;
 	priv->gconf_conn_id = 0;
+	priv->is_in_user_list = FALSE;
+	priv->user_monitor = NULL;
+	priv->recent_monitor = NULL;
 }
 
 static void
@@ -376,6 +392,9 @@ document_tile_finalize (GObject *g_object)
 
 	g_object_unref (priv->recent_monitor);
 
+	if (priv->user_monitor)
+		gnome_vfs_monitor_cancel (priv->user_monitor);
+
 	(* G_OBJECT_CLASS (document_tile_parent_class)->finalize) (g_object);
 }
 
@@ -480,6 +499,21 @@ create_subheader (const gchar *desc)
 	return subheader;
 }
 
+static void
+update_user_list_menu_item (DocumentTile *this)
+{
+	TileAction *action = TILE (this)->actions [DOCUMENT_TILE_ACTION_UPDATE_MAIN_MENU];
+	DocumentTilePrivate *priv = DOCUMENT_TILE_GET_PRIVATE (this);
+
+	if (! action)
+		return;
+
+	if (priv->is_in_user_list)
+		tile_action_set_menu_item_label (action, _("Remove from Favorites"));
+	else
+		tile_action_set_menu_item_label (action, _("Add to Favorites"));
+}
+
 static void
 header_size_allocate_cb (GtkWidget *widget, GtkAllocation *alloc, gpointer user_data)
 {
@@ -761,6 +795,24 @@ delete_trigger (Tile *tile, TileEvent *event, TileAction *action)
 	g_list_free (list);
 }
 
+static void
+user_docs_trigger (Tile *tile, TileEvent *event, TileAction *action)
+{
+	DocumentTile *this        = DOCUMENT_TILE             (tile);
+	DocumentTilePrivate *priv = DOCUMENT_TILE_GET_PRIVATE (this);
+
+
+	if (priv->is_in_user_list)
+		libslab_remove_user_doc (tile->uri);
+	else
+		libslab_add_user_doc (
+			tile->uri, priv->mime_type, priv->modified,
+			gnome_vfs_mime_application_get_name (priv->default_app),
+			gnome_vfs_mime_application_get_exec (priv->default_app));
+
+	update_user_list_menu_item (this);
+}
+
 static void
 send_to_trigger (Tile *tile, TileEvent *event, TileAction *action)
 {
@@ -804,7 +856,7 @@ send_to_trigger (Tile *tile, TileEvent *event, TileAction *action)
 		G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
 
 	if (error)
-		handle_g_error (&error, "error in %s", G_GNUC_FUNCTION );
+		handle_g_error (&error, "error in %s", G_STRFUNC);
 
 	g_free (cmd);
 	g_free (filename);
@@ -812,3 +864,24 @@ send_to_trigger (Tile *tile, TileEvent *event, TileAction *action)
 	g_free (basename);
 	g_strfreev (argv);
 }
+
+static void
+docs_store_monitor_cb (
+	GnomeVFSMonitorHandle *handle, const gchar *monitor_uri,
+	const gchar *info_uri, GnomeVFSMonitorEventType type, gpointer user_data)
+{
+	DocumentTile *this = DOCUMENT_TILE (user_data);
+	DocumentTilePrivate *priv = DOCUMENT_TILE_GET_PRIVATE (this);
+
+	gboolean is_in_user_list_current;
+
+
+	is_in_user_list_current = libslab_user_docs_store_has_uri (TILE (this)->uri);
+
+	if (is_in_user_list_current == priv->is_in_user_list)
+		return;
+
+	priv->is_in_user_list = is_in_user_list_current;
+
+	update_user_list_menu_item (this);
+}
